@@ -14,17 +14,14 @@ import random
 
 import ipywidgets as widgets
 import numpy as np
+import math
 from IPython.display import display
 from scipy.interpolate import interp1d
 
-from .event import Event, DelayedEvent
 from .agent import Agent
+from .event import Event
 from ..logger import log
-from ..systemdynamics import Constant
-from ..systemdynamics import Converter
-from ..systemdynamics import Flow
-from ..systemdynamics import NaryOperator
-from ..systemdynamics import Stock
+from ..systemdynamics import Constant, Converter, Flow, Biflow, NaryOperator, Stock
 
 
 ###################
@@ -69,11 +66,13 @@ class Model:
         self.equations = {}
         self.stocks = {}
         self.flows = {}
+        self.biflows = {}
         self.converters = {}
         self.constants = {}
         self.points = {}
         self.functions = {}
         self.fn = {}
+        self.equation_id = 0  # unique id used for internally generated functions
 
 
         # This is a placeholder. You may define SD model equations in your own 'instantiate_model' method and use them to generate hybrid models
@@ -514,17 +513,19 @@ class Model:
         return float(f(x))
 
 
-    def plot_lookup(self,lookup_names):
+    def plot_lookup(self,lookup_names,config=None):
         from ..util import lookup_data
         from ..visualizations import visualizer
-        from ..config import config
+
+        if not config:
+            from ..config import config
 
         lookup_names = lookup_names if type(lookup_names) is list else lookup_names.split(",")
 
         df = lookup_data(self, lookup_names)
 
 
-        return visualizer().plot(df=df,
+        return visualizer(config).plot(df=df,
                                     return_df=False,
                                     visualize_from_period=0,
                                     visualize_to_period=0,
@@ -546,6 +547,25 @@ class Model:
     ### System Dynamics (SD) / Hybrid Simulation handling. Use the following methods for Hybrid models: Agent based models that use SD equations  ##
     ################################################################################################################################################
 
+    @property
+    def equation_prefix(self):
+        """
+        An id that is unique within this model that can be used to generate unique equation names
+        :return:
+        """
+        self.equation_id += 1
+        return "bptk_"+str(self.equation_id)+"_"
+
+    def equation(self,equation, arg):
+        """
+        This method only exists for making SD-DSL models compatible with the new SDModel equation API.
+        In later steps, we might want to extend this method to allow for arrayed equations as well! However, first the DSL needs to support arrays as well!
+
+            :param equation: equaiton name
+            :param arg: t
+        """
+        return self.memoize(equation,arg)
+
     def memoize(self, equation, arg):
         """
         Memoize method - used by the system dynamics equations to remember values that have already been calculated.
@@ -585,7 +605,7 @@ class Model:
     def stock(self, name):
         """
         Create a SD stock
-            :param name: name of the flow
+            :param name: name of the stock
             :return: Stock object
         """
         if name in self.stocks:
@@ -609,6 +629,19 @@ class Model:
 
         return self.functions[name]
 
+    def biflow(self, name):
+        """
+        Create a SD biflow
+            :param name: Name of the biflow
+            :return: Biflow object
+        """
+        if name in self.biflows:
+            return self.biflows[name]
+        else:
+            flow = Biflow(self, name)
+            self.biflows[name] = flow
+            return flow
+
     def flow(self, name):
         """
         Create a SD flow
@@ -655,7 +688,7 @@ class Model:
             :param t: timestep to evaluate for
             :return: float of simulation result
         """
-        return self.equations[name](t)
+        return self.memoize(name,t)
 
     def reset_cache(self):
         """
